/**
 * MCS Media Computer Software
 * Copyright 2019 by Wilfried Klaas
 * Project: SPSEmulator
 * File: TestRESTEndpoints.java
 * EMail: W.Klaas@gmx.de
 * Created: 03.02.2019 wklaa_000
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package de.mcs.tools.sps.emulator;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.IOException;
import java.io.InputStream;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Set;

import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.emulator.client.SPSEmulatorClient;
import de.mcs.tools.sps.emulator.model.CommandModel.COMMAND;
import de.mcs.tools.sps.emulator.model.WebSessionModel;

/**
 * @author wklaa_000
 *
 */
class TestRESTEndpoints {

  private static SPSEmulatorClient client;

  @BeforeAll
  public static void init() throws KeyManagementException, NoSuchAlgorithmException {
    client = new SPSEmulatorClient("https://127.0.0.1:8443/emulator");
  }

  /**
   * @throws java.lang.Exception
   */
  @BeforeEach
  void setUp() throws Exception {
  }

  @Test
  void test() throws IOException {

    WebSessionModel webSessionModel = client.getNewWebsession();
    assertNotNull(webSessionModel);
    assertFalse(webSessionModel.isError());
    System.out.println(webSessionModel.toString());

    try (InputStream inputStream = ClassLoader.getSystemResourceAsStream("test.tps")) {
      List<String> readLines = IOUtils.readLines(inputStream, "UTF-8");
      assertNotNull(readLines);
      webSessionModel.getProgram().setSource(readLines.toArray(new String[0]));
    }

    Set<COMMAND> availableCommands = webSessionModel.getCommand().getAvailableCommands();
    assertNotNull(availableCommands);
    assertTrue(availableCommands.contains(COMMAND.COMPILE));
    webSessionModel.getCommand().setActualCommand(COMMAND.COMPILE);

    webSessionModel = client.processWebSession(webSessionModel);
    assertNotNull(webSessionModel);
    assertFalse(webSessionModel.isError());
    assertTrue(webSessionModel.getProgram().getBin().length > 10);
    System.out.println(webSessionModel.toString());

    assertTrue(availableCommands.contains(COMMAND.START));
    webSessionModel.getCommand().setActualCommand(COMMAND.START);
    webSessionModel = client.processWebSession(webSessionModel);
    assertNotNull(webSessionModel);
    assertFalse(webSessionModel.isError());
    assertFalse(webSessionModel.getProgram().isModified());
    System.out.println(webSessionModel.toString());
  }

}
