/*
 * Copyright 2000-2017 Vaadin Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package de.mcs.tools.sps.ui.views.emulatorview;

import java.util.List;

import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.checkbox.CheckboxGroup;
import com.vaadin.flow.component.checkbox.CheckboxGroupVariant;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.grid.Grid.SelectionMode;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.notification.Notification;
import com.vaadin.flow.component.notification.Notification.Position;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.textfield.TextField;
import com.vaadin.flow.router.PageTitle;
import com.vaadin.flow.router.Route;

import de.mcs.tools.sps.backend.Category;
import de.mcs.tools.sps.backend.CategoryService;
import de.mcs.tools.sps.backend.Review;
import de.mcs.tools.sps.backend.ReviewService;
import de.mcs.tools.sps.ui.MainLayout;
import de.mcs.tools.sps.ui.common.AbstractEditorDialog;

/**
 * Displays the list of available categories, with a search filter as well as buttons to add a new category or edit
 * existing ones.
 */
@Route(value = "emulator2", layout = MainLayout.class)
@PageTitle("SPS Emulator")
public class EmulatorView2 extends VerticalLayout {

  private final Button btStart = new Button(new Icon(VaadinIcon.STEP_FORWARD));
  private final Button btPause = new Button(new Icon(VaadinIcon.PAUSE));
  private final Button btStop = new Button(new Icon(VaadinIcon.STOP));
  private final Grid<Category> grid = new Grid<>();

  private final CheckboxGroup<String> cbgInput = new CheckboxGroup<>();
  private final CheckboxGroup<String> cbgBtn = new CheckboxGroup<>();
  private final TextField tfAdc1 = new TextField("ADC 1");
  private final TextField tfAdc2 = new TextField("ADC 2");
  private final CheckboxGroup<String> cbgOutput = new CheckboxGroup<>();

  public EmulatorView2() {
    initView();

    addToolBar();
    addContent();

    updateView();
  }

  private void initView() {
    addClassName("emulator-view");
    setDefaultHorizontalComponentAlignment(Alignment.STRETCH);
  }

  private void addToolBar() {
    HorizontalLayout viewToolbar = new HorizontalLayout();
    viewToolbar.addClassName("view-toolbar");
    viewToolbar.add(btStart);
    viewToolbar.add(btPause);
    viewToolbar.add(btStop);
    add(viewToolbar);
  }

  private void addContent() {
    HorizontalLayout mainContainer = new HorizontalLayout();

    VerticalLayout sourceContainer = new VerticalLayout();
    sourceContainer.setClassName("view-container");
    sourceContainer.setAlignItems(Alignment.STRETCH);

    grid.addColumn(Category::getName).setHeader("No.").setWidth("8em").setResizable(true);
    grid.addColumn(this::getReviewCount).setHeader("Mnemonic").setWidth("6em");
    grid.setSelectionMode(SelectionMode.NONE);

    sourceContainer.add(grid);
    mainContainer.add(sourceContainer);

    VerticalLayout emulatorContainer = new VerticalLayout();
    HorizontalLayout inputContainer = new HorizontalLayout();
    inputContainer.setClassName("emulator-view_input-border");
    cbgInput.setItems("Input 1", "Input 2", "Input 3", "Input 4");
    cbgInput.addThemeVariants(CheckboxGroupVariant.LUMO_VERTICAL);

    inputContainer.add(cbgInput);
    // inputContainer.setWidth("50%");

    VerticalLayout secondRowContainer = new VerticalLayout();

    cbgBtn.setItems("SEL", "PRG");
    cbgBtn.addThemeVariants(CheckboxGroupVariant.LUMO_VERTICAL);

    tfAdc1.setWidth("80px");
    tfAdc2.setWidth("80px");
    secondRowContainer.add(cbgBtn);
    secondRowContainer.add(tfAdc1);
    secondRowContainer.add(tfAdc2);
    inputContainer.add(secondRowContainer);
    emulatorContainer.add(inputContainer);

    HorizontalLayout outputContainer = new HorizontalLayout();
    cbgOutput.setItems("Output 1", "Output 2", "Output 3", "Output 4");
    cbgOutput.addThemeVariants(CheckboxGroupVariant.LUMO_VERTICAL);

    outputContainer.add(cbgOutput);
    emulatorContainer.add(outputContainer);

    mainContainer.add(emulatorContainer);
    add(mainContainer);
  }

  private String getReviewCount(Category category) {
    List<Review> reviewsInCategory = ReviewService.getInstance().findReviews(category.getName());
    int sum = reviewsInCategory.stream().mapToInt(Review::getCount).sum();
    return Integer.toString(sum);
  }

  private void updateView() {
    // List<Category> categories =
    // CategoryService.getInstance().findCategories(searchField.getValue());
    // grid.setItems(categories);
    //
    // if (searchField.getValue().length() > 0) {
    // header.setText("Search for “" + searchField.getValue() + "”");
    // } else {
    // header.setText("Categories");
    // }
  }

  private void saveCategory(Category category, AbstractEditorDialog.Operation operation) {
    CategoryService.getInstance().saveCategory(category);

    Notification.show("Category successfully " + operation.getNameInText() + "ed.", 3000, Position.BOTTOM_START);
    updateView();
  }

  private void deleteCategory(Category category) {
    List<Review> reviewsInCategory = ReviewService.getInstance().findReviews(category.getName());

    reviewsInCategory.forEach(review -> {
      review.setCategory(CategoryService.getInstance().getUndefinedCategory());
      ReviewService.getInstance().saveReview(review);
    });
    CategoryService.getInstance().deleteCategory(category);

    Notification.show("Category successfully deleted.", 3000, Position.BOTTOM_START);
    updateView();
  }
}
