package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestBSRV {

  private static final int BYTE_VALUE_PWM = 0xf4;
  private static final int BYTE_VALUE_SRV = 0xf6;
  private final String MNEMONIC = "BSTA";

  @Test
  void testMnemonicPWM() throws SyntaxError {
    BSTA mnemonic = new BSTA(MNEMONIC + " PWM1");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_PWM, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new BSTA(MNEMONIC + " PWM2");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_PWM + 1, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      BSTA mno1 = new BSTA(MNEMONIC + " PWM3");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      BSTA mno1 = new BSTA(MNEMONIC + " PWM0");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicSRV() throws SyntaxError {
    BSTA mnemonic = new BSTA(MNEMONIC + " SRV1");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_SRV, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new BSTA(MNEMONIC + " SRV2");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_SRV + 1, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      BSTA mno1 = new BSTA(MNEMONIC + " SRV3");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      BSTA mno1 = new BSTA(MNEMONIC + " SRV0");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " PWM1", 0);
    assertEquals(BSTA.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " SRV1", 0);
    assertEquals(BSTA.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 5", 0);
    });
  }
}
