package de.mcs.tools.sps;
/**
 * 
 */


import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import de.mcs.tools.sps.annotations.SPSOutputter;
import de.mcs.tools.sps.mnemonic.Mnemonic;

/**
 * @author w.klaas
 *
 */
@SPSOutputter(name = "TPSTXT")
public class TPSTextOutputter implements Outputter {

  /**
   * 
   */
  public TPSTextOutputter() {
  }

  /* (non-Javadoc)
   * @see de.mcs.tools.sps.Outputter#output(java.util.List, java.io.OutputStream)
   */
  @Override
  public void output(List<Mnemonic> mnemonics, OutputStream output) {
    try (Writer writer = new BufferedWriter(new OutputStreamWriter(output))) {
      writer.write("Addr   BD   Befehl   Daten     Kommentar\r\n");
      int address = 0;
      for (Mnemonic mnemonic : mnemonics) {
        byte lowNibble = (byte) (mnemonic.getByte() & 0x0f);
        byte highNibble = (byte) ((mnemonic.getByte() >> 4) & 0x0f);
        writer.write(String.format("0x%03x  %02x   %4s     %4s      %s %s\r\n", address, mnemonic.getByte(),
            nibbleToString(highNibble), nibbleToString(lowNibble), mnemonic.getName(),
            StringUtils.isEmpty(mnemonic.getArgument()) ? "" : mnemonic.getArgument()));
        address++;
      }
    } catch (IOException e) {
      e.printStackTrace();
    } finally {
    }
  }

  private String nibbleToString(byte lowNibble) {
    StringBuilder b = new StringBuilder();
    b.append((lowNibble & 0x08) > 0 ? "X" : "0");
    b.append((lowNibble & 0x04) > 0 ? "X" : "0");
    b.append((lowNibble & 0x02) > 0 ? "X" : "0");
    b.append((lowNibble & 0x01) > 0 ? "X" : "0");
    return b.toString();
  }

  @Override
  public String getDefaultExtension() {
    return ".txt";
  }

}
